/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.core;

import java.util.*;

/**
 * This class corresponds to the &lt;CTCap&gt; tag in the SyncML devinf 
 * DTD v1.1.x
 *
 * @version $Id: CTCapV1.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public final class CTCapV1 implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    private ArrayList ctTypeSupported = new ArrayList();

    // ------------------------------------------------------------ Constructors
    /**
     * In order to expose the server configuration like WS this constructor
     * must be public
     */
    public CTCapV1() {}

    /**
     * Creates a new CTCap object with the given array of information
     *
     * @param ctTypeSupported the array of information on content type
     *                        capabilities - NOT NULL
     */
    public CTCapV1(final CTTypeSupported[] ctTypeSupported) {
        setCTTypeSupported(ctTypeSupported);
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Get an array of content type information objects
     *
     * @return an array of content type information objects
     */
    public ArrayList getCTTypeSupported() {
        return this.ctTypeSupported;
    }

    /**
     * Sets an array of content type information objects
     *
     * @param ctTypeSupported an array of content type information objects
     */
    public void setCTTypeSupported(CTTypeSupported[] ctTypeSupported) {

        if (ctTypeSupported == null || ctTypeSupported.length == 0) {
            throw new IllegalArgumentException("ctTypeSupported cannot be null");
        }
        this.ctTypeSupported.clear();
        this.ctTypeSupported.addAll(Arrays.asList(ctTypeSupported));
    }
    
    /**
     * Sets an array of content type information objects
     *
     * @param ctTypeSupported an array of content type information objects
     */
    public void setCTTypeSupported(ArrayList ctTypeSupported) {

        if (ctTypeSupported == null || ctTypeSupported.size() == 0) {
            throw new IllegalArgumentException("ctTypeSupported cannot be null");
        }
        this.ctTypeSupported.clear();
        this.ctTypeSupported.addAll(ctTypeSupported);
    }
}