/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.core;

import java.util.Arrays;
import java.util.ArrayList;

/**
 * Corresponds to the &lt;Data&gt; tag in the SyncML represent DTD when
 * delivering an Anchor or a DevInf or a Property
 *
 * @version $Id: ComplexData.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public final class ComplexData
extends Data
implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    private Anchor   anchor  ;
    private DevInf   devInf  ;
    private ArrayList properties = new ArrayList();

    // ------------------------------------------------------------ Constructors
    /** For serialization purposes */
    public ComplexData() {}

    /**
     * Creates a Data object from the given anchors string.
     *
     * @param data the data
     *
     */
    public ComplexData(String data) {
        super(data);
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Gets the Anchor object property
     *
     * @return anchor the Anchor object
     */
    public Anchor getAnchor() {
        return anchor;
    }

    /**
     * Sets the Anchor object property
     *
     * @param anchor the Anchor object
     */
    public void setAnchor(Anchor anchor) {
        if (null == anchor) {
            throw new IllegalArgumentException("anchor cannot be null");
        }
        this.anchor = anchor;
    }

    /**
     * Gets the DevInf object property
     *
     * @return devInf the DevInf object property
     */
    public DevInf getDevInf() {
        return devInf;
    }

    /**
     * Sets the DevInf object property
     *
     * @param devInf the DevInf object property
     *
     */
    public void setDevInf(DevInf devInf) {
        if (null == devInf) {
            throw new IllegalArgumentException("devInf cannot be null");
        }
        this.devInf = devInf;
    }

    /**
     * Get an array of supported properties of a given content type
     *
     * @return an array of supported properties
     */
    public ArrayList getProperties() {
        return this.properties;
    }

    /**
     * Sets an array of supported properties of a given content type
     *
     * @param properties an array of supported properties
     */
    public void setProperties(Property[] properties) {

        if (properties == null || properties.length == 0) {
            throw new IllegalArgumentException("properties cannot be null");
        }
        this.properties.clear();
        this.properties.addAll(Arrays.asList(properties));
    }
}
