/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
package com.funambol.framework.core;

/**
 * This class represents the &lt;Move&gt; tag as defined by the SyncML 
 * representation specifications.
 *
 * @version $Id: Move.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public final class Move 
extends ItemizedCommand 
implements java.io.Serializable {
    
    // --------------------------------------------------------------- Constants
    public static String COMMAND_NAME = "Move";
    
    // ------------------------------------------------------------ Constructors
    
    /**
     * For serialization purposes
     */
    protected Move() {}
    
    /**
     * Creates a new Move object given its elements.
     *
     * @param cmdID the command identifier - NOT NULL
     * @param noResp is &lt;NoResponse/&gt; required?
     * @param cred the authentication credentials
     * @param meta the meta information
     * @param items an array of Item element - NOT NULL
     *
     * @throws IllegalArgumentException if any NOT NULL parameter is null
     */
    public Move(final CmdID cmdID   ,
                final boolean noResp, 
                final Cred cred     , 
                final Meta meta     , 
                final Item[] items  ) {
        super(cmdID, meta, items);
             
        setCred(cred);
        this.noResp = (noResp) ? new Boolean(noResp) : null;
    }
        
   // ----------------------------------------------------------- Public methods
        
    /**
     * Returns the command name
     *
     * @return the command name
     */
    public String getName() {
        return Put.COMMAND_NAME;
    }
}