/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
package com.funambol.framework.core;

/**
 * This class represents to the &lt;TargetParent&gt; s defined by the SyncML 
 * representation specifications 
 *
 * @version $Id: TargetParent.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public final class TargetParent implements java.io.Serializable {
    // ------------------------------------------------------------ Private data

    /**
     * Specifies the target specific URI
     */
    private String locURI;    
    
    // ------------------------------------------------------------ Constructors
    
    /**
     * For serialization purposes
     */
    protected TargetParent() {}
    
    /**
     * Creates a new TargetParent object given its URI.
     *
     * @param locURI the target URI - NOT NULL
     *
     * @throws IllegalArgumentException if locURI is null.
     */
    public TargetParent(final String locURI) {
        setLocURI(locURI);
    }
            
    // ------------------------------------------------------ Public methods
    
    /**
     * Returns the target URI value
     *
     * @return the target URI value
     */
    public String getLocURI() {
        return locURI;
    }
    
    /**
     * Sets the target URI
     *
     * @param locURI the target URI - NOT NULL
     *
     * @throws IllegalArgumentException if locURI is null
     */
    public void setLocURI(final String locURI) {
        if (locURI == null) {
            throw new IllegalArgumentException("locURI cannot be null");
        }
        this.locURI = locURI;
    }    
}