/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.engine.source;

import java.security.Principal;
import java.sql.Timestamp;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.filter.FilterClause;


/**
 * A <i>SyncSource</i> that supports the filtering.
 *
 *
 * @version $Id: FilteredSyncSource.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public interface FilteredSyncSource extends SyncSource {

    // ---------------------------------------------------------- Public Methods

    /**
     * Called before any other synchronization method. To interrupt the sync
     * process, throw a SyncSourceException.
     *
     * @param syncMode the synchronization mode being performed
     * @param filter the filter to use in the sync
     *
     * @throws SyncSourceException to interrupt the process with an error
     */
    void beginSync(Principal principal,
                   int syncMode,
                   FilterClause filter) throws SyncSourceException;

    /**
     * Called to check if the given item satisfies the filter clause specified
     * in the beginSync
     * @param item the item to check
     * @return true is the item satisfies the fiter, false otherwise
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem item) throws SyncSourceException;


    /**
     * Called to check if the item with the given key satisfies the filter
     * clause specified in the beginSync
     * @param item the item to check
     * @return true is the item satisfies the fiter, false otherwise
     * @throws SyncSourceException if the item with the given key doesn't exist or
     *                             if an error occurs
     */
    public boolean isSyncItemInFilterClause(SyncItemKey key) throws SyncSourceException;
}
