/**
 * Copyright (C) 2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.engine.transformer;


/**
 * This class represents a data encoder or decoder
 *
 * A DataTransformer is a component that given some data in some format
 * as input, it generates a transformation of the source data in output.
 * Examples of transformations are:
 *
 * - base64 encoding/decoding
 * - encryption/decription
 */
public abstract class DataTransformer {
    // ------------------------------------------------------------ Private data

    private String name;

    // ------------------------------------------------------------ Constructors


    /** Creates a new instance of DataTransformer */
    public DataTransformer(String name) {
        this.name = name;
    }

    // ---------------------------------------------------------- Public methods

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    // -------------------------------------------------------- Abstract methdos

    public abstract byte[] transform(byte[] data, TransformationInfo info)
           throws TransformerException ;

}
