/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.framework.notification;

import com.funambol.framework.core.Alert;

/**
 * This class is the principal component of the server alerted synchronization
 * architecture.
 * It is responsible for: creating the binary package and  sending the message
 * over the protocol and medium to be used.
 *
 * @version  $Id: NotificationEngine.java,v 1.1 2006/01/18 17:00:07 luigiafassina Exp $
 */
public interface NotificationEngine {

    // ---------------------------------------------------------- Public Methods
    /**
     * Sends a notification message to the given device.
     *
     * @param deviceId device identifier
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    void sendNotificationMessage(String  deviceId,
                                 Alert[] alerts  ,
                                 int     uimode  )
    throws NotificationException;

    /**
      * Sends a notification message to all devices of the principals with the
      * given username.
      *
      * @param username the username
      * @param alerts   array of Alert command for the datastore to sync, used
      *                 to create the binary message to send
      * @param uimode   user intraction mode
      */
    void sendNotificationMessages(String  username,
                                  Alert[] alerts  ,
                                  int     uimode  )
    throws NotificationException;
}
