/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.server.admin.ejb;

import javax.ejb.*;

import com.funambol.framework.config.LoggingConfiguration;
import com.funambol.framework.core.Alert;
import com.funambol.framework.engine.source.SyncSource;
import com.funambol.framework.filter.Clause;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Capabilities;
import com.funambol.framework.server.error.ServerException;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.Sync4jModule;
import com.funambol.framework.server.Sync4jUser;

import com.funambol.server.admin.AdminException;
import com.funambol.server.config.Configuration;

/**
 * @version $Id: AdminLocal.java,v 1.2 2006/01/20 14:30:23 nichele Exp $
 */
public interface AdminLocal extends EJBLocalObject {

    /**
     * CONFIGURATION
     */
    public Configuration getConfig();

    /**
     * USER
     */
    public String[] getRoles()
    throws ServerException, AdminException;

    public Sync4jUser[] getUsers(Clause clause)
    throws ServerException, AdminException;

    public void addUser(Sync4jUser u)
    throws ServerException, AdminException;

    public void setUser(Sync4jUser u)
    throws ServerException, AdminException;

    public void deleteUser(String userName)
    throws ServerException, AdminException;

    public int countUsers(Clause clause)
    throws ServerException, AdminException;


    /**
     * DEVICE
     */
    public Sync4jDevice[] getDevices(Clause clause)
    throws ServerException, AdminException;

    public String addDevice(Sync4jDevice d)
    throws ServerException, AdminException;

    public void setDevice(Sync4jDevice d)
    throws ServerException, AdminException;

    public void deleteDevice(String deviceId)
    throws ServerException, AdminException;

    public int countDevices(Clause clause)
    throws ServerException, AdminException;

    /**
     * CAPABILITIES
     */
    public Capabilities getDeviceCapabilities(String deviceId)
    throws ServerException, AdminException;

    public Integer setDeviceCapabilities(Capabilities caps, String deviceId)
    throws ServerException, AdminException;


    /**
     * PRINCIPAL
     */
    public Sync4jPrincipal[] getPrincipals(Clause clause)
    throws ServerException, AdminException;

    public String addPrincipal(Sync4jPrincipal p)
    throws ServerException, AdminException;

    public void deletePrincipal(String principalId)
    throws ServerException, AdminException;

    public int countPrincipals(Clause clause)
    throws ServerException, AdminException;


    /**
     * MODULE
     */
    public Sync4jModule[] getModulesName()
    throws ServerException, AdminException;

    public Sync4jModule getModule(String moduleId)
    throws ServerException, AdminException;

    /**
     * SOURCE
     */
    public void addSource(String moduleId, String connectorId, String sourceTypeId, SyncSource source)
    throws ServerException, AdminException;

    public void setSource(String moduleId, String connectorId, String sourceTypeId, SyncSource source)
    throws ServerException, AdminException;

    public void deleteSource(String sourceUri)
    throws ServerException, AdminException;

    /**
     * SETTINGS
     */

    /**
     * NOTE: do not use this method as it will be replaced by
     * getServerConfiguration() in next version
     */
    public LoggingConfiguration getLoggingConfiguration()
    throws ServerException, AdminException;

    /**
     * SERVER BEANS
     */

    /**
     * Returns the server bean with the given name. If the bean does not exist,
     * am AdminException is thrown.
     *
     * @param bean the server bean
     *
     * @throws ServerException, AdminException
     */
    public Object getServerBean(String bean)
    throws ServerException, AdminException;

    /**
     * Sets the server bean with the given name.
     *
     * @param bean the server bean
     * @param obj the bean instance
     *
     * @throws ServerException, AdminException
     */
    public void setServerBean(String bean, Object obj)
    throws ServerException, AdminException;

    /**
     * Sends a notification message to the given device.
     *
     * @param deviceId device identifier
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    public void sendNotificationMessage(String  deviceId,
                                        Alert[] alerts  ,
                                        int     uimode  )
    throws ServerException;

    /**
     * Sends a notification message to all devices of the principals with the
     * given username.
     *
     * @param username the username
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    public void sendNotificationMessages(String  username,
                                         Alert[] alerts  ,
                                         int     uimode  )
    throws ServerException;
}
