/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.server.notification.builder;

import com.funambol.framework.core.Alert;
import com.funambol.framework.notification.NotificationException;
import com.funambol.framework.notification.builder.*;

/**
 * This class is an implementation of DSNotificationMessageBuilder
 *
 * @see DSNotificationMessageBuilder
 *
 * @version $Id: DSNotificationMessageBuilderImpl.java,v 1.1 2006/01/18 17:00:11 luigiafassina Exp $
 */
public class DSNotificationMessageBuilderImpl
implements DSNotificationMessageBuilder {
    
    // -------------------------------------------------------------- Constructor
    public DSNotificationMessageBuilderImpl() {
    }
    
    // ---------------------------------------------------------- Public Methods
    /**
     * Builds the notification message with the given information
     *
     * @param sessionId       the identifier of the session associated with the 
     *                        Notification Package
     * @param deviceAddress   the address or phone number (depending on the 
     *                        implementation) of the device where the message 
     *                        should be sent
     * @param serverId        the id of the server
     * @param serverPw        server's password
     * @param serverNonce     server's nonce
     * @param alerts          array of Alert command for the datastore to sync, 
     *                        used to create the binary message to send
     * @param protocolVersion the version of the SyncML protocol
     *
     * @return array of byte containing the notification message
     * @throws NotificationException
     */
    public byte[] buildMessage(int     sessionId      ,
                               String  deviceAddress  ,
                               String  serverId       ,
                               String  serverPw       ,
                               byte[]  serverNonce    ,
                               Alert[] alerts         ,
                               int     uimode         ,
                               float   protocolVersion)
    throws NotificationException {
        
        //message digest
        DigestNotificationMessage digestMessage = new DigestNotificationMessage();
        digestMessage.setNonce(serverNonce);
        digestMessage.setServerId(serverId);
        digestMessage.setServerPw(serverPw);
        
        //message header
        HeaderMessage header = new HeaderMessage();
        header.setInitiator(HeaderMessage.INITIATOR_SERVER);
        
        header.setUiMode(uimode);
        
        header.setVersion(protocolVersion);
        header.setServerIdentifier(serverId);
        header.setSessionId(sessionId);
        
        //body
        BodyMessage bodyMessage = new DSBodyMessage(alerts);
        
        //new message
        NotificationMessage notificationMsg = new NotificationMessage();
        notificationMsg.setDigest(digestMessage);
        notificationMsg.setHeader(header);
        notificationMsg.setBody(bodyMessage);
        
        byte[] msg = notificationMsg.computeTriggerNotificationMessage();
        
        return msg;
    }
}
