/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307  USA
 */

package com.funambol.server.notification.sender;

import java.io.FileOutputStream;

import com.funambol.framework.notification.NotificationException;
import com.funambol.server.notification.sender.WAPSender;

/**
 * This is a simple implementation that just writes the message in a file.
 * @version $Id: SimpleWAPSenderImpl.java,v 1.2 2006/01/30 17:15:42 nichele Exp $
 */
public class SimpleWAPSenderImpl extends WAPSender {

    /**
     * Sends the notification message to a device.
     * @param deviceAddress address of the device where message should be sent
     * @param message notification message with wap header that has to be sent
     * @throws NotificationException
     */
    public void sendMessage(String deviceAddress, byte[] message) throws NotificationException
    {
        writeByte(message, deviceAddress);
    }

    /**
     * Writes an array of bytes into a file
     * @param message bytes to be written in file
     * @param phoneNumber String used to create the file name
     */
    private void writeByte(byte[] message, String deviceAddress) {
    	if (deviceAddress == null || deviceAddress.length() == 0) {
    		deviceAddress = "Unknown";
    	}
        try
        {
            FileOutputStream fout = new FileOutputStream(deviceAddress + "_" + System.currentTimeMillis() + ".message");
            fout.write(message);
            fout.flush();
            fout.close();
        }
        catch (Exception ex)
        {
            // ignore this exception
        }
    }

}
