/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.server.security;

import java.util.HashMap;
import java.util.logging.Logger;
import java.util.logging.Level;

import java.security.Principal;
import javax.security.auth.*;
import javax.security.auth.login.*;
import javax.security.*;

import com.funambol.framework.filter.*;

import com.funambol.framework.security.Officer;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.logging.Sync4jLogger;
import com.funambol.framework.core.Cred;
import com.funambol.framework.core.Authentication;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.config.ConfigurationConstants;
import com.funambol.framework.tools.MD5;
import com.funambol.framework.tools.Base64;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;

import com.funambol.framework.config.ConfigurationConstants;
import com.funambol.server.config.Configuration;
import com.funambol.server.admin.UserManager;

/**
 * This is an implementation of the <i>Officier</i> interface. It always
 * authenticates and authorizes users and resource accesses.
 *
 * @version $Id: DBOfficer.java,v 1.1 2006/01/18 17:00:11 luigiafassina Exp $
 */
public class DBOfficer implements Officer, java.io.Serializable {

    // ---------------------------------------------------------- Protected data

    protected transient Logger log    = Sync4jLogger.getLogger();
    protected PersistentStore ps      = null;
    protected UserManager userManager = null;

    // ------------------------------------------------------------- Public data
    // -------------------------------------------------------------- Properties

    /**
     * Has the last login failed for incorrect login/password?
     */
    private boolean loginFailed = false;

    public boolean isLoginFailed() {
        return loginFailed;
    }

    /**
     * Has the last login failed for expired temporary login?
     */
    private boolean loginExpired = false;

    /**
     * Which type of authetication does impose the server?
     */
    private String clientAuth = Cred.AUTH_TYPE_BASIC;

    public String getClientAuth() {
        return this.clientAuth;
    }
    public void setClientAuth(String clientAuth) {
        this.clientAuth = clientAuth;
    }

    /**
     * Which type of authetication use the server to send his credential?
     */
    private String serverAuth = Cred.AUTH_NONE;
    public String getServerAuth() {
        return this.serverAuth;
    }

    public void setServerAuth(String serverAuth) {
        this.serverAuth = serverAuth;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     *
     * @return true if the credential is autenticated, false otherwise
     */
    public boolean authenticate(Cred credential) {
        Configuration config = Configuration.getConfiguration();
        ps = config.getStore();

        userManager = (UserManager)config.getUserManager();

        String type = credential.getType();

        if ((Cred.AUTH_TYPE_BASIC).equals(type)) {
            return authenticateBasicCredential(credential);
        } else if ( (Cred.AUTH_TYPE_MD5).equals(type)  ) {
            String syncMLVerProto =
                             credential.getAuthentication().getSyncMLVerProto();
            //
            // The modality in order to calculate the digest is different to second
            // of the used version of SyncML
            //
            if (syncMLVerProto.indexOf("1.0") != -1) {
                return authenticateMD5Credential10(credential);
            } else if (syncMLVerProto.indexOf("1.1") != -1) {
                return authenticateMD5Credential11(credential);
            }
        } else if ( (Cred.AUTH_TYPE_HMAC).equals(type)  ) {
            return false;
        }
        return false;
    }

    private boolean authenticateBasicCredential(Cred credential) {
        Sync4jUser[] users = null;
        String username = null, password = null;
        Sync4jPrincipal principal = null;

        Authentication auth = credential.getAuthentication();
        String deviceId     = auth.getDeviceId();
        String userpwd      = new String(Base64.decode(auth.getData()));

        int p = userpwd.indexOf(':');

        if (p == -1) {
            username = userpwd;
            password = "";
        } else {
            username = (p>0) ? userpwd.substring(0, p) : "";
            password = (p == (userpwd.length()-1)) ? "" : userpwd.substring(p+1);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Username: " + username);
        }

        //
        // Verify that exist the principal for the specify deviceId and username
        //
        try {
            principal = new Sync4jPrincipal(username, deviceId);
            ps.read(principal);
            credential.getAuthentication().setPrincipalId(principal.getId());
        } catch(PersistentStoreException e) {
            log.severe("Error reading principal: " + e);
            log.throwing(getClass().getName(), "authenticateBasicCredential", e);
            return false;
        }

        try {
            WhereClause[] wc = new WhereClause[2];
            String value[] = new String[1];
            value[0] = username;
            wc[0] = new WhereClause("username", value, WhereClause.OPT_EQ, true);

            value = new String[1];
            value[0] = password;
            wc[1] = new WhereClause("password", value, WhereClause.OPT_EQ, true);
            LogicalClause lc = new LogicalClause(LogicalClause.OPT_AND, wc);

            users = userManager.getUsers(lc);
            if (users.length == 1) {
                return true;
            }

        } catch(PersistentStoreException e) {
            log.severe("Error reading user: " + e);
            log.throwing(getClass().getName(), "authenticateBasicCredential", e);
            return false;
        }


        return false;
    }
    private boolean authenticateMD5Credential10(Cred credential) {
        Sync4jPrincipal principals[] = null ;
        boolean isAuthenticate       = false;
        String password              = null ;
        String username              = null ;
        String usernamepwd           = null ;

        Authentication auth = credential.getAuthentication();
        String deviceId = auth.getDeviceId();

        //
        // Gets all principal for the specify deviceId
        //
        WhereClause whereClause = new WhereClause("device",
                                                  new String[] {deviceId},
                                                  WhereClause.OPT_EQ,
                                                  true);
        try {
            principals = (Sync4jPrincipal[])ps.read(
                new Sync4jPrincipal(null, deviceId),
                whereClause
            );
        } catch(PersistentStoreException e) {
            log.severe("Error reading principals: " + e);
            log.throwing(getClass().getName(), "authenticateMD5Credential11", e);
            return false;
        }
        //
        // Search the username
        //
        int size = principals.length;
        for (int i=0; i<size; i++) {
            Sync4jUser user = new Sync4jUser(principals[i].getUsername(), null, null, null, null, null);

            try {
                userManager.getUser(user);
            } catch(PersistentStoreException e) {
                log.severe("Error reading user: " + e);
                log.throwing(getClass().getName(), "authenticateMD5Credential11", e);
            }

            //
            // Calculate digest
            //
            username = user.getUsername();
            password = user.getPassword();
            usernamepwd = username + ':' + password;
            byte[] usernamepwdBytes = usernamepwd.getBytes();

            byte[] clientNonce = auth.getNextNonce().getValue();

            if (log.isLoggable(Level.FINEST)) {
                log.finest("username: " + username);
                log.finest("password: " + password);
                log.finest("clientNonce  : " + new String(Base64.encode(clientNonce)));
            }

            //
            // computation of the MD5 digest
            //
            // Creates a unique buffer containing the bytes to digest
            //
            byte[] buf = new byte[usernamepwdBytes.length + 1 + clientNonce.length];

            System.arraycopy(usernamepwdBytes, 0, buf, 0, usernamepwdBytes.length);
            buf[usernamepwdBytes.length] = (byte)':';
            System.arraycopy(clientNonce, 0, buf, usernamepwdBytes.length+1, clientNonce.length);

            byte[] digest = MD5.digest(buf);

            //
            // encoding digest in Base64 for comparation with digest sent by client
            //
            String serverDigestNonceB64 = new String(Base64.encode(digest));

            //
            // digest sent by client in format b64
            //
            String msgDigestNonceB64 = auth.getData();

            if (log.isLoggable(Level.FINEST)) {
                log.finest("serverDigestNonceB64: " + serverDigestNonceB64);
                log.finest("msgDigestNonceB64: "    + msgDigestNonceB64   );
            }

            if (msgDigestNonceB64.equals(serverDigestNonceB64)) {
                credential.getAuthentication().setPrincipalId(principals[i].getId());
                isAuthenticate = true;
                break;
            }
        }

        if (!isAuthenticate) {
            return false;
        }

        auth.setUsername(usernamepwd);
        credential.setAuthentication(auth);
        return true;
    }

    private boolean authenticateMD5Credential11(Cred credential) {
        Sync4jPrincipal principals[] = null ;
        boolean isAuthenticate       = false;
        String password              = null ;
        String username              = null ;
        byte[] userDigestB64         = null ;

        Authentication auth = credential.getAuthentication();
        String deviceId = auth.getDeviceId();

        //
        // Gets all principal for the specify deviceId
        //
        WhereClause whereClause = new WhereClause("device",
                                                  new String[] {deviceId},
                                                  WhereClause.OPT_EQ,
                                                  true);
        try {
            principals = (Sync4jPrincipal[])ps.read(
                new Sync4jPrincipal(null, deviceId),
                whereClause
            );
        } catch(PersistentStoreException e) {
            log.severe("Error reading principals: " + e);
            log.throwing(getClass().getName(), "authenticateMD5Credential11", e);
            return false;
        }
        //
        // Search the username
        //
        int size = principals.length;
        for (int i=0; i<size; i++) {
            Sync4jUser user = new Sync4jUser(principals[i].getUsername(), null, null, null, null, null);

            try {
                userManager.getUser(user);
            } catch(PersistentStoreException e) {
                log.severe("Error reading user: " + e);
                log.throwing(getClass().getName(), "authenticateMD5Credential11", e);
            }

            //
            // Calculate digest
            //
            username = user.getUsername();
            password = user.getPassword();

            byte[] userDigest = MD5.digest((new String(username + ':' + password)).getBytes());
            userDigestB64 = Base64.encode(userDigest);

            byte[] clientNonce  = auth.getNextNonce().getValue();

            if (log.isLoggable(Level.FINEST)) {
                log.finest("userDigestB64: " + new String(userDigestB64));
                log.finest("clientNonce  : " + new String(Base64.encode(clientNonce)));
            }

            //
            // computation of the MD5 digest
            //
            // Creates a unique buffer containing the bytes to digest
            //
            byte[] buf = new byte[userDigestB64.length + 1 + clientNonce.length];

            System.arraycopy(userDigestB64, 0, buf, 0, userDigestB64.length);
            buf[userDigestB64.length] = (byte)':';
            System.arraycopy(clientNonce, 0, buf, userDigestB64.length+1, clientNonce.length);

            byte[] digest = MD5.digest(buf);

            //
            // encoding digest in Base64 for comparation with digest sent by client
            //
            String serverDigestNonceB64 = new String(Base64.encode(digest));

            //
            // digest sent by client in format b64
            //
            String msgDigestNonceB64 = auth.getData();

            if (log.isLoggable(Level.FINEST)) {
                log.finest("serverDigestNonceB64: " + serverDigestNonceB64);
                log.finest("msgDigestNonceB64: "    + msgDigestNonceB64   );
            }

            if (msgDigestNonceB64.equals(serverDigestNonceB64)) {
                credential.getAuthentication().setPrincipalId(principals[i].getId());
                isAuthenticate = true;
                break;
            }
        }

        if (!isAuthenticate) {
            return false;
        }
        auth.setUsername(new String(userDigestB64));
        credential.setAuthentication(auth);
        return true;
    }

    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource the name (or the identifier) of the resource to be authorized
     *
     * @return true if the credential is authorized to access the resource, false
     *         otherwise
     */
    public boolean authorize(Principal principal, String resource) {
        return true;
    }

    /** Un-authenticates a credential.
     *
     * @param credential the credential to be unauthenticated
     */
    public void unAuthenticate(Cred credential) {
        //
        // Do nothing. In the current implementation, the authentication is
        // discarde as soon as the LoginContext is garbage collected.
        //
    }

    /**
     * @see com.funambol.framework.security.Officer
     */
    public boolean isAccountExpired() {
        return loginExpired;
    }


    private void readObject(java.io.ObjectInputStream in)
    throws java.io.IOException, ClassNotFoundException {
        in.defaultReadObject();
        log = Sync4jLogger.getLogger();
    }
}
